----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch
--
-- Module Name: scalp_aurora_phy - arch
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.2
-- Description: scalp_aurora_phy
--
-- Last update: 2020-10-13
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.axi4_pkg.all;
use work.aurora_status_pkg.all;
use work.aurora_drp_pkg.all;

entity scalp_aurora_phy is

    generic (
        C_DEBUG_MODE : boolean := false);

    port (
        -- GTP Ref Clocks
        GTRefClkxCI      : in  t_gt_ref_slave_clk;
        -- Aurora System and GTP Clocks
        AuroraClkxCI     : in  t_aurora_slave_clk;
        AuroraClkxCO     : out t_aurora_master_clk;
        -- Aurora Reset
        AuroraResetxRI   : in  t_aurora_slave_reset;
        AuroraResetxRO   : out t_aurora_master_link_reset;
        -- GTP Serial IO
        -- North
        GTPFromNorthxDI  : in  t_aurora_gtp_diff_io_rx;
        GTPToNorthxDO    : out t_aurora_gtp_diff_io_tx;
        -- East
        GTPFromEastxDI   : in  t_aurora_gtp_diff_io_rx;
        GTPToEastxDO     : out t_aurora_gtp_diff_io_tx;
        -- South
        GTPFromSouthxDI  : in  t_aurora_gtp_diff_io_rx;
        GTPToSouthxDO    : out t_aurora_gtp_diff_io_tx;
        -- West
        GTPFromWestxDI   : in  t_aurora_gtp_diff_io_rx;
        GTPToWestxDO     : out t_aurora_gtp_diff_io_tx;
        -- Axi4 Framing Interface
        -- North
        NorthRXM2SxDO    : out t_axi4m2s;
        NorthTXM2SxDI    : in  t_axi4m2s;
        NorthTXS2MxDO    : out t_axi4s2m;
        -- East
        EastRXM2SxDO     : out t_axi4m2s;
        EastTXM2SxDI     : in  t_axi4m2s;
        EastTXS2MxDO     : out t_axi4s2m;
        -- South
        SouthRXM2SxDO    : out t_axi4m2s;
        SouthTXM2SxDI    : in  t_axi4m2s;
        SouthTXS2MxDO    : out t_axi4s2m;
        -- West
        WestRXM2SxDO     : out t_axi4m2s;
        WestTXM2SxDI     : in  t_axi4m2s;
        WestTXS2MxDO     : out t_axi4s2m;
        -- Axi4 Framing UFC Interface
        -- North
        NorthRXUFCM2SxDO : out t_axi4ufcm2s_rx;
        NorthTXUFCM2SxDI : in  t_axi4ufcm2s_tx;
        NorthTXUFCS2MxDO : out t_axi4ufcs2m_tx;
        -- East
        EastRXUFCM2SxDO  : out t_axi4ufcm2s_rx;
        EastTXUFCM2SxDI  : in  t_axi4ufcm2s_tx;
        EastTXUFCS2MxDO  : out t_axi4ufcs2m_tx;
        -- South
        SouthRXUFCM2SxDO : out t_axi4ufcm2s_rx;
        SouthTXUFCM2SxDI : in  t_axi4ufcm2s_tx;
        SouthTXUFCS2MxDO : out t_axi4ufcs2m_tx;
        -- West
        WestRXUFCM2SxDO  : out t_axi4ufcm2s_rx;
        WestTXUFCM2SxDI  : in  t_axi4ufcm2s_tx;
        WestTXUFCS2MxDO  : out t_axi4ufcs2m_tx;
        -- Aurora Ctrl + Status
        AuroraCtrlxDI    : in  t_aurora_control;
        AuroraStatusxDO  : out t_aurora_status;
        -- DRP Port
        AuroraDRPM2SxDI  : in  t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0);
        AuroraDRPS2MxDO  : out t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0));

end scalp_aurora_phy;


architecture arch of scalp_aurora_phy is

    constant C_SIM_GTRESET_SPEEDUP : string  := "false";
    constant C_NUM_PLL             : integer := 1;

    component reset_logic is
        port (
            UserClkxCI     : in  std_logic;
            InitClkxCI     : in  std_logic;
            ResetxRI       : in  std_logic;
            GTResetxRI     : in  std_logic;
            SystemResetxRO : out std_logic;
            GTResetxRO     : out std_logic);
    end component reset_logic;

    component clock_module is
        generic (
            C_NUM_PLL : integer);
        port (
            InitClkxCI      : in  std_logic;
            InitClkxCO      : out std_logic;
            GTClkxCI        : in  std_logic_vector((C_NUM_PLL - 1) downto 0);
            GTClkLockedxDI  : in  std_logic_vector((C_NUM_PLL - 1) downto 0);
            UserClkxCO      : out std_logic_vector((C_NUM_PLL - 1) downto 0);
            SyncClkxCO      : out std_logic_vector((C_NUM_PLL - 1) downto 0);
            PLLNotLockedxDO : out std_logic_vector((C_NUM_PLL - 1) downto 0));
    end component clock_module;

    component gt_common is
        generic (
            C_SIM_GTRESET_SPEEDUP : string);
        port (
            GT0GTRefClk0xCI      : in  std_logic;
            GT0PLL0LockxSO       : out std_logic;
            GT0PLL1LockxSO       : out std_logic;
            GT0PLL0LockDetClkxCI : in  std_logic;
            GT0PLL0RefClkLostxSO : out std_logic;
            GT0PLL0ClkxCO        : out std_logic;
            GT0PLL1ClkxCO        : out std_logic;
            GT0PLL0RefClkxCO     : out std_logic;
            GT0PLL1RefClkxCO     : out std_logic;
            GT0PLL0ResetxRI      : in  std_logic);
    end component gt_common;

    -- From Aurora Clock Module
    signal AuroraSlaveClkxC        : t_aurora_slave_clk                            := C_AURORA_NO_SLAVE_CLK;
    signal AuroraMasterClkxC       : t_aurora_master_clk                           := C_AURORA_NO_MASTER_CLK;
    -- From GT Common Module
    signal GTCommonSlaveClkxC      : t_gt_common_slave_clk                         := C_GT_COMMON_NO_SLAVE_CLK;
    signal GTCommonMasterClkxC     : t_gt_common_master_clk                        := C_GT_COMMON_NO_MASTER_CLK;
    -- From Aurora Reset Logic
    signal AuroraSlaveResetxR      : t_aurora_slave_reset                          := C_AURORA_NO_SLAVE_RESET;
    signal AuroraMasterResetxR     : t_aurora_master_reset                         := C_AURORA_NO_MASTER_RESET;
    -- From Aurora Core
    signal AuroraMasterLinkResetxR : t_aurora_master_link_reset                    := C_AURORA_NO_MASTER_LINK_RESET;
    signal NorthRXM2SxD            : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal NorthTXM2SxD            : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal NorthTXS2MxD            : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal EastRXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal EastTXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal EastTXS2MxD             : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal SouthRXM2SxD            : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal SouthTXM2SxD            : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal SouthTXS2MxD            : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal WestRXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal WestTXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal WestTXS2MxD             : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal NorthRXUFCM2SxD         : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal NorthTXUFCM2SxD         : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal NorthTXUFCS2MxD         : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal EastRXUFCM2SxD          : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal EastTXUFCM2SxD          : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal EastTXUFCS2MxD          : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal SouthRXUFCM2SxD         : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal SouthTXUFCM2SxD         : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal SouthTXUFCS2MxD         : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal WestRXUFCM2SxD          : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal WestTXUFCM2SxD          : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal WestTXUFCS2MxD          : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal GTPFromNorthxD          : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToNorthxD            : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromEastxD           : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToEastxD             : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromSouthxD          : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToSouthxD            : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromWestxD           : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToWestxD             : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal AuroraCtrlxD            : t_aurora_control                              := C_AURORA_NO_CONTROL;
    signal AuroraStatusxD          : t_aurora_status                               := C_AURORA_NO_STATUS;
    signal AuroraDRPM2SxD          : t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0) := (others => C_NO_DRP_M2S);
    signal AuroraDRPS2MxD          : t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0) := (others => C_NO_DRP_S2M);

begin

    ClocksIOxB : block is
    begin  -- block ClocksIOxB

        AuroraClkSlaveInitClkxAS       : AuroraSlaveClkxC.InitClkxC  <= AuroraClkxCI.InitClkxC;
        AuroraClkSlavexDrpClkxAS       : AuroraSlaveClkxC.DrpClkxC   <= AuroraClkxCI.DrpClkxC;
        AuroraClkMasterInitClkxAS      : AuroraClkxCO.InitClkxC      <= AuroraMasterClkxC.InitClkxC;
        AuroraClkMasterUserClkxAS      : AuroraClkxCO.UserClkxC      <= AuroraMasterClkxC.UserClkxC;
        AuroraClkMasterPllNotLockedxAS : AuroraClkxCO.PLLNotLockedxS <= AuroraMasterClkxC.PllNotLockedxS;

    end block ClocksIOxB;

    ResetIOxB : block is
    begin  -- block ResetIOxB

        GT0GTRefClk0xAS                     : GTCommonSlaveClkxC.GTRefClkxC         <= GTRefClkxCI.GTRefClkxC;
        GT0PLL0LockDetClkxAS                : GTCommonSlaveClkxC.GTPLL0LockDetClkxC <= AuroraMasterClkxC.InitClkxC;
        AuroraSlaveResetResetxAS            : AuroraSlaveResetxR.ResetxR            <= AuroraResetxRI.ResetxR;
        AuroraSlaveResetGTResetxAS          : AuroraSlaveResetxR.GTResetxR          <= AuroraResetxRI.GTResetxR;
        AuroraMasterLinkResetSystemResetxAS : AuroraResetxRO.SystemResetxR          <= AuroraMasterLinkResetxR.SystemResetxR;
        AuroraMasterLinkResetLinkResetxAS   : AuroraResetxRO.LinkResetxR            <= AuroraMasterLinkResetxR.LinkResetxR;

    end block ResetIOxB;

    SignalsIOxB : block is
    begin  -- block SignalsIOxB

        -- Axi4 Framing
        NorthRXM2SxAS    : NorthRXM2SxDO    <= NorthRXM2SxD;
        NorthTXM2SxAS    : NorthTXM2SxD     <= NorthTXM2SxDI;
        NorthTXS2MxAS    : NorthTXS2MxDO    <= NorthTXS2MxD;
        EastRXM2SxAS     : EastRXM2SxDO     <= EastRXM2SxD;
        EastTXM2SxAS     : EastTXM2SxD      <= EastTXM2SxDI;
        EastTXS2MxAS     : EastTXS2MxDO     <= EastTXS2MxD;
        SouthRXM2SxAS    : SouthRXM2SxDO    <= SouthRXM2SxD;
        SouthTXM2SxAS    : SouthTXM2SxD     <= SouthTXM2SxDI;
        SouthTXS2MxAS    : SouthTXS2MxDO    <= SouthTXS2MxD;
        WestRXM2SxAS     : WestRXM2SxDO     <= WestRXM2SxD;
        WestTXM2SxAS     : WestTXM2SxD      <= WestTXM2SxDI;
        WestTXS2MxAS     : WestTXS2MxDO     <= WestTXS2MxD;
        -- Axi4 Framing UFC
        NorthRXUFCM2SxAS : NorthRXUFCM2SxDO <= NorthRXUFCM2SxD;
        NorthTXUFCM2SxAS : NorthTXUFCM2SxD  <= NorthTXUFCM2SxDI;
        NorthTXUFCS2MxAS : NorthTXUFCS2MxDO <= NorthTXUFCS2MxD;
        EastRXUFCM2SxAS  : EastRXUFCM2SxDO  <= EastRXUFCM2SxD;
        EastTXUFCM2SxAS  : EastTXUFCM2SxD   <= EastTXUFCM2SxDI;
        EastTXUFCS2MxAS  : EastTXUFCS2MxDO  <= EastTXUFCS2MxD;
        SouthRXUFCM2SxAS : SouthRXUFCM2SxDO <= SouthRXUFCM2SxD;
        SouthTXUFCM2SxAS : SouthTXUFCM2SxD  <= SouthTXUFCM2SxDI;
        SouthTXUFCS2MxAS : SouthTXUFCS2MxDO <= SouthTXUFCS2MxD;
        WestRXUFCM2SxAS  : WestRXUFCM2SxDO  <= WestRXUFCM2SxD;
        WestTXUFCM2SxAS  : WestTXUFCM2SxD   <= WestTXUFCM2SxDI;
        WestTXUFCS2MxAS  : WestTXUFCS2MxDO  <= WestTXUFCS2MxD;
        -- GTP Serial IO
        GTPFromNorthxAS  : GTPFromNorthxD   <= GTPFromNorthxDI;
        GTPToNorthxAS    : GTPToNorthxDO    <= GTPToNorthxD;
        GTPFromEastxAS   : GTPFromEastxD    <= GTPFromEastxDI;
        GTPToEastxAS     : GTPToEastxDO     <= GTPToEastxD;
        GTPFromSouthxAS  : GTPFromSouthxD   <= GTPFromSouthxDI;
        GTPToSouthxAS    : GTPToSouthxDO    <= GTPToSouthxD;
        GTPFromWestxAS   : GTPFromWestxD    <= GTPFromWestxDI;
        GTPToWestxAS     : GTPToWestxDO     <= GTPToWestxD;
        -- Aurora Ctrl + Status
        AuroraCtrlxAS    : AuroraCtrlxD     <= AuroraCtrlxDI;
        AuroraStatusxAS  : AuroraStatusxDO  <= AuroraStatusxD;
        -- Aurora DRP Port
        AuroraDRPM2SxAS  : AuroraDRPM2SxD   <= AuroraDRPM2SxDI;
        AuroraDRPS2MxAS  : AuroraDRPS2MxDO  <= AuroraDRPS2MxD;

    end block SignalsIOxB;

    ClkRstxB : block is
    begin  -- block ClkRstxB

        AuroraResetLogicxI : entity work.reset_logic
            port map (
                UserClkxCI     => AuroraMasterClkxC.UserClkxC,
                InitClkxCI     => AuroraMasterClkxC.InitClkxC,
                ResetxRI       => AuroraSlaveResetxR.ResetxR,  -- Sync on UserClk
                GTResetxRI     => AuroraSlaveResetxR.GTResetxR,  -- Sync on InitClk
                SystemResetxRO => AuroraMasterResetxR.SystemResetxR,
                GTResetxRO     => AuroraMasterResetxR.GTResetxR);

        AuroraClockModulexI : entity work.clock_module
            generic map (
                C_NUM_PLL => C_NUM_PLL)
            port map (
                -- Slave
                InitClkxCI         => AuroraSlaveClkxC.InitClkxC,
                GTClkxCI(0)        => AuroraSlaveClkxC.GTClkxC,
                GTClkLockedxDI(0)  => AuroraSlaveClkxC.GTClkLockedxS,
                -- Master
                InitClkxCO         => AuroraMasterClkxC.InitClkxC,
                UserClkxCO(0)      => AuroraMasterClkxC.UserClkxC,
                SyncClkxCO(0)      => AuroraMasterClkxC.SyncClkxC,
                PLLNotLockedxDO(0) => AuroraMasterClkxC.PllNotLockedxS);

        GTCommonxI : entity work.gt_common
            generic map (
                C_SIM_GTRESET_SPEEDUP => C_SIM_GTRESET_SPEEDUP)
            port map (
                -- Slave
                GT0GTRefClk0xCI      => GTCommonSlaveClkxC.GTRefClkxC,
                GT0PLL0LockDetClkxCI => GTCommonSlaveClkxC.GTPLL0LockDetClkxC,
                GT0PLL0ResetxRI      => GTCommonSlaveClkxC.CommonResetxR,
                -- Master
                GT0PLL0ClkxCO        => GTCommonMasterClkxC.GTPLL0ClkxCO,
                GT0PLL1ClkxCO        => GTCommonMasterClkxC.GTPLL1ClkxCO,
                GT0PLL0RefClkxCO     => GTCommonMasterClkxC.GTPLL0RefClkxC,
                GT0PLL1RefClkxCO     => GTCommonMasterClkxC.GTPLL1RefClkxC,
                GT0PLL0LockxSO       => GTCommonMasterClkxC.GTPLL0LockxS,
                GT0PLL1LockxSO       => GTCommonMasterClkxC.GTPLL1LockxS,
                GT0PLL0RefClkLostxSO => GTCommonMasterClkxC.GTPLL0RefClkLostxS);

    end block ClkRstxB;

    -- Aurora GT Phy

    AuroraGTPhyxB : block is
    begin  -- block AuroraGTPhyxB

        -- North Channel

        NorthChannelxI : entity work.north_channel
            port map (
                -- Clocks and Locked
                -- Slave
                gt_refclk1            => GTCommonSlaveClkxC.GTRefClkxC,
                quad1_common_lock_in  => GTCommonMasterClkxC.GTPLL0LockxS,
                tx_out_clk            => AuroraSlaveClkxC.GTClkxC,  -- Only North
                tx_lock               => AuroraSlaveClkxC.GTClkLockedxS,  -- Only North
                drpclk_in             => AuroraSlaveClkxC.DrpClkxC,
                init_clk_in           => AuroraMasterClkxC.InitClkxC,
                -- Master            
                user_clk              => AuroraMasterClkxC.UserClkxC,
                sync_clk              => AuroraMasterClkxC.SyncClkxC,
                pll_not_locked        => AuroraMasterClkxC.PllNotLockedxS,
                -- GT0
                gt0_pll0outclk_in     => GTCommonMasterClkxC.GTPLL0ClkxCO,
                gt0_pll1outclk_in     => GTCommonMasterClkxC.GTPLL1ClkxCO,
                gt0_pll0outrefclk_in  => GTCommonMasterClkxC.GTPLL0RefClkxC,
                gt0_pll1outrefclk_in  => GTCommonMasterClkxC.GTPLL1RefClkxC,
                gt0_pll0refclklost_in => GTCommonMasterClkxC.GTPLL0RefClkLostxS,
                gt_common_reset_out   => GTCommonSlaveClkxC.CommonResetxR,  -- Only North
                -- Reset
                reset                 => AuroraMasterResetxR.SystemResetxR,
                gt_reset              => AuroraMasterResetxR.GTResetxR,
                link_reset_out        => AuroraMasterLinkResetxR.LinkResetxR(0),
                sys_reset_out         => AuroraMasterLinkResetxR.SystemResetxR(0),
                -- TX
                s_axi_tx_tdata        => NorthTXM2SxD.DataxD,
                s_axi_tx_tkeep        => NorthTXM2SxD.KeepxD,
                s_axi_tx_tlast        => NorthTXM2SxD.LastxS,
                s_axi_tx_tvalid       => NorthTXM2SxD.ValidxS,
                s_axi_tx_tready       => NorthTXS2MxD.ReadyxS,
                -- RX
                m_axi_rx_tdata        => NorthRXM2SxD.DataxD,
                m_axi_rx_tkeep        => NorthRXM2SxD.KeepxD,
                m_axi_rx_tlast        => NorthRXM2SxD.LastxS,
                m_axi_rx_tvalid       => NorthRXM2SxD.ValidxS,
                -- TX UFC
                s_axi_ufc_tx_tvalid   => NorthTXUFCM2SxD.ValidxS,
                s_axi_ufc_tx_tdata    => NorthTXUFCM2SxD.DataxD,    -- 3 bits
                s_axi_ufc_tx_tready   => NorthTXUFCS2MxD.ReadyxS,
                -- RX UFC
                m_axi_ufc_rx_tdata    => NorthRXUFCM2SxD.DataxD,    -- 32 bits
                m_axi_ufc_rx_tkeep    => NorthRXUFCM2SxD.KeepxD,
                m_axi_ufc_rx_tlast    => NorthRXUFCM2SxD.LastxS,
                m_axi_ufc_rx_tvalid   => NorthRXUFCM2SxD.ValidxS,
                -- GTP
                txp                   => GTPToNorthxD.TXPxD,
                txn                   => GTPToNorthxD.TXNxD,
                rxp                   => GTPFromNorthxD.RXPxD,
                rxn                   => GTPFromNorthxD.RXNxD,
                -- Status
                hard_err              => AuroraStatusxD.HardErrxD(0),
                soft_err              => AuroraStatusxD.SoftErrxD(0),
                frame_err             => AuroraStatusxD.FrameErrxD(0),
                channel_up            => AuroraStatusxD.ChannelUpxD(0),
                lane_up               => AuroraStatusxD.LaneUpxD(0),
                tx_resetdone_out      => AuroraStatusxD.TXResetDoneOutxD(0),
                rx_resetdone_out      => AuroraStatusxD.RXResetDoneOutxD(0),
                -- Ctrl
                loopback              => AuroraCtrlxD.LoopbackxD,
                power_down            => AuroraCtrlxD.PowerDownxS,
                -- DRP
                drpen_in              => AuroraDRPM2SxD(0).DrpEnxS,
                drpwe_in              => AuroraDRPM2SxD(0).DrpWExS,
                drpaddr_in            => AuroraDRPM2SxD(0).DrpAddrxD,
                drpdi_in              => AuroraDRPM2SxD(0).DrpDIxD,
                drprdy_out            => AuroraDRPS2MxD(0).DrpRdyxS,
                drpdo_out             => AuroraDRPS2MxD(0).DrpDOxD);

        -- East Channel

        EastChannelxI : entity work.east_channel
            port map (
                -- Clocks and Locked
                -- Slave
                gt_refclk1            => GTCommonSlaveClkxC.GTRefClkxC,
                quad1_common_lock_in  => GTCommonMasterClkxC.GTPLL0LockxS,
                tx_out_clk            => open,
                tx_lock               => open,
                drpclk_in             => AuroraSlaveClkxC.DrpClkxC,
                init_clk_in           => AuroraMasterClkxC.InitClkxC,
                -- Master            
                user_clk              => AuroraMasterClkxC.UserClkxC,
                sync_clk              => AuroraMasterClkxC.SyncClkxC,
                pll_not_locked        => AuroraMasterClkxC.PllNotLockedxS,
                -- GT0
                gt0_pll0outclk_in     => GTCommonMasterClkxC.GTPLL0ClkxCO,
                gt0_pll1outclk_in     => GTCommonMasterClkxC.GTPLL1ClkxCO,
                gt0_pll0outrefclk_in  => GTCommonMasterClkxC.GTPLL0RefClkxC,
                gt0_pll1outrefclk_in  => GTCommonMasterClkxC.GTPLL1RefClkxC,
                gt0_pll0refclklost_in => GTCommonMasterClkxC.GTPLL0RefClkLostxS,
                gt_common_reset_out   => open,
                -- Reset
                reset                 => AuroraMasterResetxR.SystemResetxR,
                gt_reset              => AuroraMasterResetxR.GTResetxR,
                link_reset_out        => AuroraMasterLinkResetxR.LinkResetxR(1),
                sys_reset_out         => AuroraMasterLinkResetxR.SystemResetxR(1),
                -- TX
                s_axi_tx_tdata        => EastTXM2SxD.DataxD,
                s_axi_tx_tkeep        => EastTXM2SxD.KeepxD,
                s_axi_tx_tlast        => EastTXM2SxD.LastxS,
                s_axi_tx_tvalid       => EastTXM2SxD.ValidxS,
                s_axi_tx_tready       => EastTXS2MxD.ReadyxS,
                -- RX
                m_axi_rx_tdata        => EastRXM2SxD.DataxD,
                m_axi_rx_tkeep        => EastRXM2SxD.KeepxD,
                m_axi_rx_tlast        => EastRXM2SxD.LastxS,
                m_axi_rx_tvalid       => EastRXM2SxD.ValidxS,
                -- TX UFC
                s_axi_ufc_tx_tvalid   => EastTXUFCM2SxD.ValidxS,
                s_axi_ufc_tx_tdata    => EastTXUFCM2SxD.DataxD,  -- 3 bits
                s_axi_ufc_tx_tready   => EastTXUFCS2MxD.ReadyxS,
                -- RX UFC
                m_axi_ufc_rx_tdata    => EastRXUFCM2SxD.DataxD,  -- 32 bits
                m_axi_ufc_rx_tkeep    => EastRXUFCM2SxD.KeepxD,
                m_axi_ufc_rx_tlast    => EastRXUFCM2SxD.LastxS,
                m_axi_ufc_rx_tvalid   => EastRXUFCM2SxD.ValidxS,
                -- GTP
                txp                   => GTPToEastxD.TXPxD,
                txn                   => GTPToEastxD.TXNxD,
                rxp                   => GTPFromEastxD.RXPxD,
                rxn                   => GTPFromEastxD.RXNxD,
                -- Status
                hard_err              => AuroraStatusxD.HardErrxD(1),
                soft_err              => AuroraStatusxD.SoftErrxD(1),
                frame_err             => AuroraStatusxD.FrameErrxD(1),
                channel_up            => AuroraStatusxD.ChannelUpxD(1),
                lane_up               => AuroraStatusxD.LaneUpxD(1),
                tx_resetdone_out      => AuroraStatusxD.TXResetDoneOutxD(1),
                rx_resetdone_out      => AuroraStatusxD.RXResetDoneOutxD(1),
                -- Ctrl
                loopback              => AuroraCtrlxD.LoopbackxD,
                power_down            => AuroraCtrlxD.PowerDownxS,
                -- DRP
                drpen_in              => AuroraDRPM2SxD(1).DrpEnxS,
                drpwe_in              => AuroraDRPM2SxD(1).DrpWExS,
                drpaddr_in            => AuroraDRPM2SxD(1).DrpAddrxD,
                drpdi_in              => AuroraDRPM2SxD(1).DrpDIxD,
                drprdy_out            => AuroraDRPS2MxD(1).DrpRdyxS,
                drpdo_out             => AuroraDRPS2MxD(1).DrpDOxD);

        -- South Channel

        SouthChannelxI : entity work.south_channel
            port map (
                -- Clocks and Locked
                -- Slave
                gt_refclk1            => GTCommonSlaveClkxC.GTRefClkxC,
                quad1_common_lock_in  => GTCommonMasterClkxC.GTPLL0LockxS,
                tx_out_clk            => open,
                tx_lock               => open,
                drpclk_in             => AuroraSlaveClkxC.DrpClkxC,
                init_clk_in           => AuroraMasterClkxC.InitClkxC,
                -- Master            
                user_clk              => AuroraMasterClkxC.UserClkxC,
                sync_clk              => AuroraMasterClkxC.SyncClkxC,
                pll_not_locked        => AuroraMasterClkxC.PllNotLockedxS,
                -- GT0
                gt0_pll0outclk_in     => GTCommonMasterClkxC.GTPLL0ClkxCO,
                gt0_pll1outclk_in     => GTCommonMasterClkxC.GTPLL1ClkxCO,
                gt0_pll0outrefclk_in  => GTCommonMasterClkxC.GTPLL0RefClkxC,
                gt0_pll1outrefclk_in  => GTCommonMasterClkxC.GTPLL1RefClkxC,
                gt0_pll0refclklost_in => GTCommonMasterClkxC.GTPLL0RefClkLostxS,
                gt_common_reset_out   => open,
                -- Reset
                reset                 => AuroraMasterResetxR.SystemResetxR,
                gt_reset              => AuroraMasterResetxR.GTResetxR,
                link_reset_out        => AuroraMasterLinkResetxR.LinkResetxR(2),
                sys_reset_out         => AuroraMasterLinkResetxR.SystemResetxR(2),
                -- TX
                s_axi_tx_tdata        => SouthTXM2SxD.DataxD,
                s_axi_tx_tkeep        => SouthTXM2SxD.KeepxD,
                s_axi_tx_tlast        => SouthTXM2SxD.LastxS,
                s_axi_tx_tvalid       => SouthTXM2SxD.ValidxS,
                s_axi_tx_tready       => SouthTXS2MxD.ReadyxS,
                -- RX
                m_axi_rx_tdata        => SouthRXM2SxD.DataxD,
                m_axi_rx_tkeep        => SouthRXM2SxD.KeepxD,
                m_axi_rx_tlast        => SouthRXM2SxD.LastxS,
                m_axi_rx_tvalid       => SouthRXM2SxD.ValidxS,
                -- TX UFC
                s_axi_ufc_tx_tvalid   => SouthTXUFCM2SxD.ValidxS,
                s_axi_ufc_tx_tdata    => SouthTXUFCM2SxD.DataxD,  -- 3 bits
                s_axi_ufc_tx_tready   => SouthTXUFCS2MxD.ReadyxS,
                -- RX UFC
                m_axi_ufc_rx_tdata    => SouthRXUFCM2SxD.DataxD,  -- 32 bits
                m_axi_ufc_rx_tkeep    => SouthRXUFCM2SxD.KeepxD,
                m_axi_ufc_rx_tlast    => SouthRXUFCM2SxD.LastxS,
                m_axi_ufc_rx_tvalid   => SouthRXUFCM2SxD.ValidxS,
                -- GTP
                txp                   => GTPToSouthxD.TXPxD,
                txn                   => GTPToSouthxD.TXNxD,
                rxp                   => GTPFromSouthxD.RXPxD,
                rxn                   => GTPFromSouthxD.RXNxD,
                -- Status
                hard_err              => AuroraStatusxD.HardErrxD(2),
                soft_err              => AuroraStatusxD.SoftErrxD(2),
                frame_err             => AuroraStatusxD.FrameErrxD(2),
                channel_up            => AuroraStatusxD.ChannelUpxD(2),
                lane_up               => AuroraStatusxD.LaneUpxD(2),
                tx_resetdone_out      => AuroraStatusxD.TXResetDoneOutxD(2),
                rx_resetdone_out      => AuroraStatusxD.RXResetDoneOutxD(2),
                -- Ctrl
                loopback              => AuroraCtrlxD.LoopbackxD,
                power_down            => AuroraCtrlxD.PowerDownxS,
                -- DRP
                drpen_in              => AuroraDRPM2SxD(2).DrpEnxS,
                drpwe_in              => AuroraDRPM2SxD(2).DrpWExS,
                drpaddr_in            => AuroraDRPM2SxD(2).DrpAddrxD,
                drpdi_in              => AuroraDRPM2SxD(2).DrpDIxD,
                drprdy_out            => AuroraDRPS2MxD(2).DrpRdyxS,
                drpdo_out             => AuroraDRPS2MxD(2).DrpDOxD);

        -- West Channel

        WestChannelxI : entity work.west_channel
            port map (
                -- Clocks and Locked
                -- Slave
                gt_refclk1            => GTCommonSlaveClkxC.GTRefClkxC,
                quad1_common_lock_in  => GTCommonMasterClkxC.GTPLL0LockxS,
                tx_out_clk            => open,
                tx_lock               => open,
                drpclk_in             => AuroraSlaveClkxC.DrpClkxC,
                init_clk_in           => AuroraMasterClkxC.InitClkxC,
                -- Master            
                user_clk              => AuroraMasterClkxC.UserClkxC,
                sync_clk              => AuroraMasterClkxC.SyncClkxC,
                pll_not_locked        => AuroraMasterClkxC.PllNotLockedxS,
                -- GT0
                gt0_pll0outclk_in     => GTCommonMasterClkxC.GTPLL0ClkxCO,
                gt0_pll1outclk_in     => GTCommonMasterClkxC.GTPLL1ClkxCO,
                gt0_pll0outrefclk_in  => GTCommonMasterClkxC.GTPLL0RefClkxC,
                gt0_pll1outrefclk_in  => GTCommonMasterClkxC.GTPLL1RefClkxC,
                gt0_pll0refclklost_in => GTCommonMasterClkxC.GTPLL0RefClkLostxS,
                gt_common_reset_out   => open,
                -- Reset
                reset                 => AuroraMasterResetxR.SystemResetxR,
                gt_reset              => AuroraMasterResetxR.GTResetxR,
                link_reset_out        => AuroraMasterLinkResetxR.LinkResetxR(3),
                sys_reset_out         => AuroraMasterLinkResetxR.SystemResetxR(3),
                -- TX
                s_axi_tx_tdata        => WestTXM2SxD.DataxD,
                s_axi_tx_tkeep        => WestTXM2SxD.KeepxD,
                s_axi_tx_tlast        => WestTXM2SxD.LastxS,
                s_axi_tx_tvalid       => WestTXM2SxD.ValidxS,
                s_axi_tx_tready       => WestTXS2MxD.ReadyxS,
                -- RX
                m_axi_rx_tdata        => WestRXM2SxD.DataxD,
                m_axi_rx_tkeep        => WestRXM2SxD.KeepxD,
                m_axi_rx_tlast        => WestRXM2SxD.LastxS,
                m_axi_rx_tvalid       => WestRXM2SxD.ValidxS,
                -- TX UFC
                s_axi_ufc_tx_tvalid   => WestTXUFCM2SxD.ValidxS,
                s_axi_ufc_tx_tdata    => WestTXUFCM2SxD.DataxD,  -- 3 bits
                s_axi_ufc_tx_tready   => WestTXUFCS2MxD.ReadyxS,
                -- RX UFC
                m_axi_ufc_rx_tdata    => WestRXUFCM2SxD.DataxD,  -- 32 bits
                m_axi_ufc_rx_tkeep    => WestRXUFCM2SxD.KeepxD,
                m_axi_ufc_rx_tlast    => WestRXUFCM2SxD.LastxS,
                m_axi_ufc_rx_tvalid   => WestRXUFCM2SxD.ValidxS,
                -- GTP
                txp                   => GTPToWestxD.TXPxD,
                txn                   => GTPToWestxD.TXNxD,
                rxp                   => GTPFromWestxD.RXPxD,
                rxn                   => GTPFromWestxD.RXNxD,
                -- Status
                hard_err              => AuroraStatusxD.HardErrxD(3),
                soft_err              => AuroraStatusxD.SoftErrxD(3),
                frame_err             => AuroraStatusxD.FrameErrxD(3),
                channel_up            => AuroraStatusxD.ChannelUpxD(3),
                lane_up               => AuroraStatusxD.LaneUpxD(3),
                tx_resetdone_out      => AuroraStatusxD.TXResetDoneOutxD(3),
                rx_resetdone_out      => AuroraStatusxD.RXResetDoneOutxD(3),
                -- Ctrl
                loopback              => AuroraCtrlxD.LoopbackxD,
                power_down            => AuroraCtrlxD.PowerDownxS,
                -- DRP
                drpen_in              => AuroraDRPM2SxD(3).DrpEnxS,
                drpwe_in              => AuroraDRPM2SxD(3).DrpWExS,
                drpaddr_in            => AuroraDRPM2SxD(3).DrpAddrxD,
                drpdi_in              => AuroraDRPM2SxD(3).DrpDIxD,
                drprdy_out            => AuroraDRPS2MxD(3).DrpRdyxS,
                drpdo_out             => AuroraDRPS2MxD(3).DrpDOxD);

    end block AuroraGTPhyxB;

end arch;
