----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: aurora_status_pkg
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.2
-- Description: Control and status signals for the Aurora phy.
--
-- Last update: 2020-10-06
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;

package aurora_status_pkg is

    -- Constants

    constant C_GTP_VECTOR_SIZE : integer := 1;
    constant C_LANEUP_SIZE     : integer := 1;
    constant C_LOOPBACK_SIZE   : integer := 3;
    constant C_NB_GTP_CORE     : integer := 4;
    constant C_NUM_PLL         : integer := 1;
    constant C_GTP_NORTH_ID    : integer := 0;
    constant C_GTP_EAST_ID     : integer := 1;
    constant C_GTP_SOUTH_ID    : integer := 2;
    constant C_GTP_WEST_ID     : integer := 3;

    -- Types

    type t_aurora_gtp_diff_io_rx is record
        RXPxD : std_ulogic_vector((C_GTP_VECTOR_SIZE - 1) downto 0);
        RXNxD : std_ulogic_vector((C_GTP_VECTOR_SIZE - 1) downto 0);
    end record t_aurora_gtp_diff_io_rx;

    type t_aurora_gtp_diff_io_tx is record
        TXPxD : std_ulogic_vector((C_GTP_VECTOR_SIZE - 1) downto 0);
        TXNxD : std_ulogic_vector((C_GTP_VECTOR_SIZE - 1) downto 0);
    end record t_aurora_gtp_diff_io_tx;

    constant C_AURORA_NO_GTP_DIFF_IO_RX : t_aurora_gtp_diff_io_rx := (RXPxD => (others => '0'),
                                                                      RXNxD => (others => '0'));

    constant C_AURORA_NO_GTP_DIFF_IO_TX : t_aurora_gtp_diff_io_tx := (TXPxD => (others => '0'),
                                                                      TXNxD => (others => '0'));

    type t_gtp_diff_ref_clk is record
        ClkPxC : std_ulogic;
        ClkNxC : std_ulogic;
    end record t_gtp_diff_ref_clk;

    constant C_NO_GTP_DIFF_REF_CLK : t_gtp_diff_ref_clk := (ClkPxC => '0',
                                                            ClkNxC => '0');

    type t_aurora_control is record
        LoopbackxD      : std_ulogic_vector((C_LOOPBACK_SIZE - 1) downto 0);
        PowerDownxS     : std_ulogic;
        GTPRefClkSrcxS  : std_ulogic;
        InitClkLockedxS : std_ulogic;
    end record t_aurora_control;

    constant C_MGTREFCLK0_SRC : std_ulogic := '0';
    constant C_MGTREFCLK1_SRC : std_ulogic := '1';

    constant C_AURORA_NO_CONTROL : t_aurora_control := (LoopbackxD      => (others => '0'),
                                                        PowerDownxS     => '1',
                                                        GTPRefClkSrcxS  => C_MGTREFCLK0_SRC,
                                                        InitClkLockedxS => '0');

    type t_laneup_vector is array (0 to (C_NB_GTP_CORE - 1)) of std_ulogic_vector((C_LANEUP_SIZE - 1) downto 0);

    type t_aurora_status is record
        HardErrxD        : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        SoftErrxD        : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        FrameErrxD       : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        LaneUpxD         : t_laneup_vector;
        ChannelUpxD      : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        RXResetDoneOutxD : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        TXResetDoneOutxD : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        TXLockxD         : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        PllNotLockedxS   : std_ulogic;
    end record t_aurora_status;

    constant C_AURORA_NO_STATUS : t_aurora_status := (HardErrxD        => (others => '0'),
                                                      SoftErrxD        => (others => '0'),
                                                      FrameErrxD       => (others => '0'),
                                                      LaneUpxD         => (others => (others => '0')),
                                                      ChannelUpxD      => (others => '0'),
                                                      RXResetDoneOutxD => (others => '0'),
                                                      TXResetDoneOutxD => (others => '0'),
                                                      TXLockxD         => (others => '0'),
                                                      PllNotLockedxS   => '1');

    constant C_AURORA_STATUS_OK : t_aurora_status := (HardErrxD        => (others => '0'),
                                                      SoftErrxD        => (others => '0'),
                                                      FrameErrxD       => (others => '0'),
                                                      LaneUpxD         => (others => (others => '1')),
                                                      ChannelUpxD      => (others => '1'),
                                                      RXResetDoneOutxD => (others => '1'),
                                                      TXResetDoneOutxD => (others => '1'),
                                                      TXLockxD         => (others => '1'),
                                                      PllNotLockedxS   => '1');

    type t_aurora_crc is record
        CRCPassFailxSN : std_ulogic;
        CRCValidxS     : std_ulogic;
    end record t_aurora_crc;

    constant C_AURORA_NO_CRC : t_aurora_crc := (CRCPassFailxSN => '0',
                                                CRCValidxS     => '0');

    constant C_AURORA_CRC_OK : t_aurora_crc := (CRCPassFailxSN => '1',
                                                CRCValidxS     => '1');

    constant C_AURORA_CRC_NO_OK : t_aurora_crc := (CRCPassFailxSN => '0',
                                                   CRCValidxS     => '1');

    type t_gt_ref_slave_clk is record
        GTRefClkxC : std_ulogic;
    end record t_gt_ref_slave_clk;

    constant C_GT_REF_NO_SLAVE_CLK : t_gt_ref_slave_clk := (GTRefClkxC => '0');

    type t_gt_common_slave_clk is record
        GTRefClkxC         : std_ulogic;
        GTPLL0LockDetClkxC : std_ulogic;
        CommonResetxR      : std_ulogic;
    end record t_gt_common_slave_clk;

    constant C_GT_COMMON_NO_SLAVE_CLK : t_gt_common_slave_clk := (GTRefClkxC         => '0',
                                                                  GTPLL0LockDetClkxC => '0',
                                                                  CommonResetxR      => '0');

    type t_gt_common_master_clk is record
        GTPLL0ClkxCO       : std_ulogic;
        GTPLL1ClkxCO       : std_ulogic;
        GTPLL0RefClkxC     : std_ulogic;
        GTPLL1RefClkxC     : std_ulogic;
        GTPLL0LockxS       : std_ulogic;
        GTPLL1LockxS       : std_ulogic;
        GTPLL0RefClkLostxS : std_ulogic;
    end record t_gt_common_master_clk;

    constant C_GT_COMMON_NO_MASTER_CLK : t_gt_common_master_clk := (GTPLL0ClkxCO       => '0',
                                                                    GTPLL1ClkxCO       => '0',
                                                                    GTPLL0RefClkxC     => '0',
                                                                    GTPLL1RefClkxC     => '0',
                                                                    GTPLL0LockxS       => '0',
                                                                    GTPLL1LockxS       => '0',
                                                                    GTPLL0RefClkLostxS => '0');

    type t_aurora_slave_clk is record
        InitClkxC     : std_ulogic;
        DrpClkxC      : std_ulogic;
        GTClkxC       : std_ulogic;
        GTClkLockedxS : std_ulogic;
    end record t_aurora_slave_clk;

    constant C_AURORA_NO_SLAVE_CLK : t_aurora_slave_clk := (InitClkxC     => '0',
                                                            DrpClkxC      => '0',
                                                            GTClkxC       => '0',
                                                            GTClkLockedxS => '0');

    type t_aurora_master_clk is record
        InitClkxC      : std_ulogic;
        UserClkxC      : std_ulogic;
        SyncClkxC      : std_ulogic;
        PllNotLockedxS : std_ulogic;
    end record t_aurora_master_clk;

    constant C_AURORA_NO_MASTER_CLK : t_aurora_master_clk := (InitClkxC      => '0',
                                                              UserClkxC      => '0',
                                                              SyncClkxC      => '0',
                                                              PllNotLockedxS => '0');

    type t_aurora_slave_reset is record
        ResetxR   : std_ulogic;
        GTResetxR : std_ulogic;
    end record t_aurora_slave_reset;

    constant C_AURORA_NO_SLAVE_RESET : t_aurora_slave_reset := (ResetxR   => '0',
                                                                GTResetxR => '0');

    type t_aurora_master_reset is record
        SystemResetxR : std_ulogic;
        GTResetxR     : std_ulogic;
    end record t_aurora_master_reset;

    constant C_AURORA_NO_MASTER_RESET : t_aurora_master_reset := (SystemResetxR => '0',
                                                                  GTResetxR     => '0');

    type t_aurora_master_link_reset is record
        SystemResetxR : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
        LinkResetxR   : std_ulogic_vector((C_NB_GTP_CORE - 1) downto 0);
    end record t_aurora_master_link_reset;

    constant C_AURORA_NO_MASTER_LINK_RESET : t_aurora_master_link_reset := (SystemResetxR => (others => '0'),
                                                                            LinkResetxR   => (others => '0'));

    -- Functions

    function std_bool (exp_in : in boolean) return std_ulogic;

end package aurora_status_pkg;

-- Functions definitions

package body aurora_status_pkg is

    function std_bool (exp_in : in boolean)
        return std_ulogic is
    begin  -- function std_bool

        if exp_in = true then
            return '1';
        else
            return '0';
        end if;

    end function std_bool;

end package body aurora_status_pkg;
